

/*
 * We latch our own power on (and off) using an MOSFET pair
 * connected in parallel with the physical (momentary contact)
 * power switch. When the power switch is pressed, one of the 
 * first things the ESP8266 needs to do is latch the power on.
 * When the current iteration of this programm is done, it
 * simply turns it's own power off again.
 *
 * ChkAutoOff() will check the run time since start-up and
 * automatically power the unit off if RUNT_MAX millis is
 * exceeded.
 */
void LatchOn() {
    pinMode(POWER_SW, OUTPUT);  /* Don't forget! */
    digitalWrite(POWER_SW, HIGH);
#ifdef DEBUG
    Serial.println("Power switch latched on.");
#endif
}


void LatchOff() {
    digitalWrite(POWER_SW, LOW);
#ifdef DEBUG
    Serial.println("Power switch latch off.");
#endif
}


void ChkAutoOff() {
    unsigned long tm_now = millis();
    if (tm_now >= RUNT_MAX) {
#ifdef DEBUG
        Serial.println("Time limit P/Off.");
#endif
        LatchOff();
    }
}


/*
 * We call LatchOn() as our first action at the very top of
 * setup().
 */
void setup() {
    int conn_tries = 0;
    char clidbuff[STRBUFF_SIZE];

    /* Before we do anything else, latch the power on. */
    LatchOn();

    /*  ALL YOUR USUAL STUFF HERE  */
}


/*
 * We call ChkAutoOff() each time through the main loop(), so
 * the unit will power itself off after a preset time.  You can
 * add a "busy" flag which you'd update from (for instance) the
 * MQTT server or any seconady user input switches, to reset the
 * timeout value if the unit is still busy processing.
 */
void loop() {
    ArduinoOTA.handle();
    yield();
    ChkAutoOff();
    yield();

    /*  DO YOUR REAL WORK HERE  */
}
